@@ -743,6 +743,7 @@ const string EnvBaseT::GetString( BaseGDL*& p, bool calledFromHELP)
 	  (p != NULL && env.Loc( ix) == p) 
 	  )
 	{
+          if (p == NullGDL::GetSingleInstance()) std::cerr<<"NULL pointer: uncertain name"<<std::endl;
 	  if( subUD != NULL) {return subUD->GetVarName(ix);}
 
 	  string callerName = Default;
@@ -766,11 +767,19 @@ const string EnvBaseT::GetString( BaseGDL*& p, bool calledFromHELP)
   //   if( sysVar != NULL) return sysVar->Name();
 
   // search common blocks
-  if( name == Default && subUD != NULL)
+  if (name == Default && subUD != NULL)
     {
       string varName;
-      if( subUD->GetCommonVarName4Help( p, varName)) 
-	return varName;
+      if (calledFromHELP)
+        {
+          if (subUD->GetCommonVarName4Help (p, varName))
+            return varName;
+        }
+      else
+        {
+          if (subUD->GetCommonVarName (p, varName))
+            return varName;
+        }
     }
 
   if( !p)
@@ -1295,9 +1304,9 @@ SizeT EnvT::NParam( SizeT minPar)
 
 bool EnvBaseT::Removeall() 
 {
-	DSubUD* proD=dynamic_cast<DSubUD*>(pro);
+DSubUD* proD=dynamic_cast<DSubUD*>(pro);
 	int osz = env.size();
-	for( SizeT ix=0; ix < osz; ix++) {
+	for( ssize_t ix=osz-1; ix >= 0; ix--) {
 		if( env[ix] != NULL) GDLDelete( env[ix]);
 		env.pop_back();
 	}
@@ -1364,8 +1373,10 @@ int EnvBaseT::findvar(BaseGDL* delP)
 
 void EnvBaseT::SetNextParUnchecked( BaseGDL* const nextP) // by value (reset loc)
 {
-  //   if(!( static_cast<int>(parIx - pro->key.size()) < pro->nPar))
-  assert( static_cast<int>(parIx - pro->key.size()) < pro->nPar);
+  if(!( static_cast<int>(parIx - pro->key.size()) < pro->nPar)){
+	  throw GDLException(callingNode,
+			     pro->Name()+": Incorrect number of arguments.",false,false);
+	}
   env.Set(parIx++,nextP); // check done in parameter_def
 }
 void EnvBaseT::SetNextParUncheckedVarNum( BaseGDL* const nextP) // by reference (reset env)
@@ -1376,7 +1387,10 @@ void EnvBaseT::SetNextParUncheckedVarNum( BaseGDL* const nextP) // by reference
 
 void EnvBaseT::SetNextParUnchecked( BaseGDL** const nextP) // by reference (reset env)
 {
-  assert( static_cast<int>(parIx - pro->key.size()) < pro->nPar);
+  if(!( static_cast<int>(parIx - pro->key.size()) < pro->nPar)){
+	  throw GDLException(callingNode,
+			     pro->Name()+": Incorrect number of arguments.",false,false);
+	}
   env.Set(parIx++,nextP);
 }
 void EnvBaseT::SetNextParUncheckedVarNum( BaseGDL** const nextP) // by reference (reset env)
