@@ -62,13 +62,21 @@
 // GDLDATADIR
 #include "config.h"
 
+//initialize wxWidgets system
+#ifdef HAVE_LIBWXWIDGETS
+#include "gdlwidget.hpp"
+#if __WXMSW__ 
+wxIMPLEMENT_APP_NO_MAIN( wxAppGDL);
+#else
+wxIMPLEMENT_APP_NO_MAIN( wxApp);
+#endif
+#endif
+
 using namespace std;
 
 static void StartupMessage()
 {
-  cerr << endl;
   cerr << "  GDL - GNU Data Language, Version " << VERSION << endl;
-  cerr << endl;
   cerr << "- For basic information type HELP,/INFO" << endl;
 }
 
@@ -88,7 +96,7 @@ void InitOpenMP() {
   suggested_num_threads=get_suggested_omp_num_threads();
   omp_num_core=omp_get_num_procs();
 
-  //  cout << "estimated Threads :" << suggested_num_threads << endl;
+  //  cerr << "estimated Threads :" << suggested_num_threads << endl;
 
   // we update iff needed (by default, "omp_num_threads" is initialiazed to "omp_num_core"
   if ((suggested_num_threads > 0) && (suggested_num_threads < omp_num_core)) {
@@ -98,9 +106,14 @@ void InitOpenMP() {
     static unsigned NTHREADSTag = cpu->Desc()->TagIndex( "TPOOL_NTHREADS");
     (*static_cast<DLongGDL*>( cpu->GetTag( NTHREADSTag, 0)))[0] =suggested_num_threads;
 
-    // effective gloabl change of num of treads using omp_set_num_threads()
+    // effective global change of num of treads using omp_set_num_threads()
+    CpuTPOOL_NTHREADS=suggested_num_threads;
     omp_set_num_threads(suggested_num_threads);
+  } else {
+    CpuTPOOL_NTHREADS=omp_get_num_procs();
+    omp_set_num_threads(CpuTPOOL_NTHREADS);
   }
+  //  cout << CpuTPOOL_NTHREADS <<endl;
 #endif
 }
 
@@ -123,8 +136,9 @@ void GDLSetLimits()
 struct rlimit* gdlstack=new struct rlimit;
   int r=getrlimit(RLIMIT_STACK,gdlstack); 
 //  cerr <<"Current rlimit = "<<gdlstack->rlim_cur<<endl;
-//  cerr<<"Max rlimit = "<<  gdlstack->rlim_max<<endl;     
-  if (gdlstack->rlim_max > GDL_PREFERED_STACKSIZE ) gdlstack->rlim_cur=GDL_PREFERED_STACKSIZE;
+//  cerr<<"Max rlimit = "<<  gdlstack->rlim_max<<endl;
+  if (gdlstack->rlim_cur >= GDL_PREFERED_STACKSIZE ) return; //the bigger the better.
+  if (gdlstack->rlim_max > GDL_PREFERED_STACKSIZE ) gdlstack->rlim_cur=GDL_PREFERED_STACKSIZE; //not completely satisfactory.
   r=setrlimit(RLIMIT_STACK,gdlstack);
 }
 #endif
@@ -134,7 +148,7 @@ void InitGDL()
 #ifndef _WIN32
   GDLSetLimits();
 #endif
-#if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
+#if defined(HAVE_LIBREADLINE)
   // initialize readline (own version - not pythons one)
   // in includefirst.hpp readline is disabled for python_module
   rl_initialize();
@@ -144,11 +158,10 @@ void InitGDL()
   //when editing the command line with ARROW keys. (bug 562). (used also in dinterpreted.cpp )
   //but... without it we have no graphics event handler! FIXME!!! 
   rl_event_hook = GDLEventHandler;
-  // SA: history is now stifled in the interpreter.InterpreterLoop( startup),
-  //     enabling one to set the history-file length via the !EDIT_INPUT sysvar
-  // stifle_history( 20);
 #endif
-
+#ifdef HAVE_LIBWXWIDGETS
+    if (useWxWidgets) GDLWidget::Init();
+#endif
   // ncurses blurs the output, initialize TermWidth here
   TermWidth();
 
@@ -178,6 +191,9 @@ static bool trace_me;
 // SA: for use in COMMAND_LINE_ARGS()
 namespace lib {
   extern std::vector<std::string> command_line_args;
+#ifdef _WIN32
+  bool posixpaths;
+#endif
   bool gdlarg_present(const char* s)
   {
 		for (size_t i = 0; i < command_line_args.size(); i++)
@@ -210,51 +226,64 @@ int main(int argc, char *argv[])
   bool strict_syntax=false;
   bool syntaxOptionSet=false;
 
-  //start with a default value:
-  useWxWidgetsForGraphics = false;
   bool force_no_wxgraphics = false;
-
+  usePlatformDeviceName=false;
+  forceWxWidgetsUglyFonts = false;
   useDSFMTAcceleration = true;
-
+  iAmANotebook=false; //option --notebook
+ #ifdef HAVE_LIBWXWIDGETS 
+  useWxWidgets=true;
+#else
+  useWxWidgets=false;
+#endif  
+#ifdef _WIN32
+  lib::posixpaths = false;
+#endif
   for( SizeT a=1; a< argc; ++a)
     {
-      if( string( argv[a]) == "--help" | string( argv[a]) == "-h")
-	{
-	  cout << "Usage: gdl [ OPTIONS ] [ batch_file ... ]" << endl;
-	  cout << "Start the GDL interpreter (incremental compiler)" << endl;
-	  cout << endl;
-	  cout << "GDL options:" << endl;
-	  cout << "  --help (-h)        display this message" << endl;
-	  cout << "  --version (-V, -v) show version information" << endl;
-	  cout << "  --fakerelease X.y  pretend that !VERSION.RELEASE is X.y" << endl;
-	  cout << "  --fussy            implies that procedures adhere with modern IDL, where \"()\" are for functions and \"[]\" are for arrays." <<endl;
-      cout << "                     This speeds up (sometimes terribly) compilation but choke on every use of \"()\" with arrays." << endl;
-      cout << "                     Conversion of procedures to modern IDL can be done with D. Landsman's idlv4_to_v5 procedure." << endl;
-      cout << "                     Use enviromnment variable \"GDL_IS_FUSSY\" to set up permanently this feature." << endl;
-	  cout << "  --sloppy           Sets the traditional (default) compiling option where \"()\"  can be used both with functions and arrays." << endl;
-      cout << "                     Needed to counteract temporarily the effect of the enviromnment variable \"GDL_IS_FUSSY\"." << endl;
-      cout << "  --use-wx           Tells GDL to use WxWidgets graphics instead of X11 or Windows. (nicer plots)." << endl;
-      cout << "                     Also enabled by setting the environment variable GDL_USE_WX to a non-null value." << endl;
-      cout << "  --no-use-wx        Tells GDL no to use WxWidgets graphics, even if env. var. \"GDL_USE_WX\" is set." << endl;
-      cout << "  --no-dSFMT         Tells GDL not to use double precision SIMD oriented Fast Mersenne Twister(dSFMT) for random doubles." << endl;
-      cout << "                     Also disable by setting the environment variable GDL_NO_DSFMT to a non-null value." << endl;
-      cout << endl;
-	  cout << "IDL-compatible options:" << endl;
-	  cout << "  -arg value tells COMMAND_LINE_ARGS() to report" << endl;
-          cout << "             the following argument (may be specified more than once)" << endl;
-	  cout << "  -args ...  tells COMMAND_LINE_ARGS() to report " << endl;
-          cout << "             all following arguments" << endl;
-          cout << "  -e value   execute given statement and exit (last occurence taken into account only," << endl;
-          cout << "             executed after startup file, may not be specified together with batch files)" << endl;
-	  cout << "  -pref=/path/to/params_file  loads the specified preference file" << endl;
-	  cout << "  -quiet (--quiet, -q) suppress welcome messages" << endl;
-	  cout << endl;
-	  cout << "Homepage: http://gnudatalanguage.sf.net" << endl;
-	  return 0;
-	}
+      if( string( argv[a]) == "--help" | string( argv[a]) == "-h") {
+      cerr << "Usage: gdl [ OPTIONS ] [ batch_file ... ]" << endl;
+      cerr << "Start the GDL interpreter (incremental compiler)" << endl;
+      cerr << endl;
+      cerr << "GDL options:" << endl;
+      cerr << "  --help (-h)        display this message" << endl;
+      cerr << "  --version (-V, -v) show version information" << endl;
+      cerr << "  --fakerelease X.y  pretend that !VERSION.RELEASE is X.y" << endl;
+      cerr << "  --fussy            implies that procedures adhere with modern IDL, where \"()\" are for functions and \"[]\" are for arrays." << endl;
+      cerr << "                     This speeds up (sometimes terribly) compilation but choke on every use of \"()\" with arrays." << endl;
+      cerr << "                     Conversion of procedures to modern IDL can be done with D. Landsman's idlv4_to_v5 procedure." << endl;
+      cerr << "                     Use enviromnment variable \"GDL_IS_FUSSY\" to set up permanently this feature." << endl;
+      cerr << "  --sloppy           Sets the traditional (default) compiling option where \"()\"  can be used both with functions and arrays." << endl;
+      cerr << "                     Needed to counteract temporarily the effect of the enviromnment variable \"GDL_IS_FUSSY\"." << endl;
+      cerr << "  --MAC              Graphic device will be called 'MAC' on MacOSX. (default: 'X')" << endl;
+      cerr << "  --no-use-wx        Tells GDL not to use WxWidgets graphics." << endl;
+      cerr << "                     Also enabled by setting the environment variable GDL_DISABLE_WX_PLOTS to a non-null value." << endl;
+      cerr << "  --notebook         Force SVG-only device, used only when GDL is a Python Notebook Kernel." << endl;
+      cerr << "  --widget-compat    Tells GDL to use a default (rather ugly) fixed pitch font for compatiblity with IDL widgets." << endl;
+      cerr << "                     Also enabled by setting the environment variable GDL_WIDGET_COMPAT to a non-null value." << endl;
+      cerr << "                     Using this option may render some historical widgets unworkable (as they are based on fixed sizes)." << endl;
+      cerr << "  --no-dSFMT         Tells GDL not to use double precision SIMD oriented Fast Mersenne Twister(dSFMT) for random doubles." << endl;
+      cerr << "                     Also disable by setting the environment variable GDL_NO_DSFMT to a non-null value." << endl;
+#ifdef _WIN32
+      cerr << "  --posix (Windows only): paths will be posix paths (experimental)." << endl;
+#endif
+      cerr << endl;
+      cerr << "IDL-compatible options:" << endl;
+      cerr << "  -arg value tells COMMAND_LINE_ARGS() to report" << endl;
+      cerr << "             the following argument (may be specified more than once)" << endl;
+      cerr << "  -args ...  tells COMMAND_LINE_ARGS() to report " << endl;
+      cerr << "             all following arguments" << endl;
+      cerr << "  -e value   execute given statement and exit (last occurrence taken into account only," << endl;
+      cerr << "             executed after startup file, may not be specified together with batch files)" << endl;
+      cerr << "  -pref=/path/to/params_file  loads the specified preference file" << endl;
+      cerr << "  -quiet (--quiet, -q) suppress welcome messages" << endl;
+      cerr << endl;
+      cerr << "Homepage: https://gnudatalanguage.github.io" << endl;
+      return 0;
+    }
       else if (string(argv[a])=="--version" | string(argv[a])=="-v" | string(argv[a])=="-V")
 	{
-	  cout << "GDL - GNU Data Language, Version " << VERSION << endl;
+	  cerr << "GDL - GNU Data Language, Version " << VERSION << endl;
 	  return 0;
 	}
       else if( string( argv[a]) == "-arg")
@@ -277,11 +306,11 @@ int main(int argc, char *argv[])
 	}
       else if (string(argv[a]).find("-pref=") ==0)
 	{
-	  cout << "This option is not operational now" << endl;
+	  cerr << "This option is not operational now" << endl;
 	  string tmp;
 	  tmp=string(argv[a]);
 	  string params_file(tmp.begin()+6,tmp.end());
-	  //cout << "(not ready) to be processed file >>" << params_file << "<<" << endl;
+	  //cerr << "(not ready) to be processed file >>" << params_file << "<<" << endl;
 	  WordExp(params_file);
 	  ifstream file_params;
 	  file_params.open(params_file.c_str());
@@ -331,13 +360,24 @@ int main(int argc, char *argv[])
       {
            useDSFMTAcceleration = false;
       }
-      else if (string(argv[a]) == "--use-wx")
+      else if (string(argv[a]) == "--widget-compat")
       {
-          useWxWidgetsForGraphics = true;
+          forceWxWidgetsUglyFonts = true;
       }      
+#ifdef _WIN32
+      else if (string(argv[a]) == "--posix") lib::posixpaths=true;
+#endif
+      else if (string(argv[a]) == "--MAC")
+      {
+         usePlatformDeviceName = true;
+      }
       else if (string(argv[a]) == "--no-use-wx")
       {
-           force_no_wxgraphics = true;
+         force_no_wxgraphics = true;
+      }
+      else if (string(argv[a]) == "--notebook")
+      {
+         iAmANotebook = true;
       }
       else if (string(argv[a]) == "--fakerelease")
       {
@@ -366,13 +406,25 @@ int main(int argc, char *argv[])
   }
   
   //before InitGDL() as InitGDL() starts graphic!
-  std::string useWX=GetEnvString("GDL_USE_WX");
-  if ( useWX.length() > 0) useWxWidgetsForGraphics=true; //not necessary "YES".
+  
+#ifdef HAVE_LIBWXWIDGETS
+  //tells if wxWidgets is working (may not be the case if DISPLAY is not set) by setting useWxWidgets to false
+  useWxWidgets=GDLWidget::InitWx();
+  // default is wx Graphics...
+  useWxWidgetsForGraphics=useWxWidgets;
+#else
+  useWxWidgetsForGraphics=false;
+#endif
+#ifdef HAVE_X
+  // unless we have X and want to see it for plots
+  std::string disableWXPlots=GetEnvString("GDL_DISABLE_WX_PLOTS");
+  if ( disableWXPlots.length() > 0) useWxWidgetsForGraphics=false; //not necessary "YES".
   if (force_no_wxgraphics) useWxWidgetsForGraphics=false; //this has the last answer, whatever the setup.
-  if (useWxWidgetsForGraphics) cerr << "- Using WxWidgets as graphics library (windows and widgets)." <<endl;
+#endif  
+  std::string doUseUglyFonts=GetEnvString("GDL_WIDGETS_COMPAT");
+  if ( doUseUglyFonts.length() > 0) forceWxWidgetsUglyFonts=true; 
   
-  
-  InitGDL();
+  InitGDL(); 
 
   // must be after !cpu initialisation
   InitOpenMP();
@@ -388,14 +440,25 @@ int main(int argc, char *argv[])
     {
       gdlPath = "+" GDLDATADIR "/lib";
       if (gdlde || (isatty(0) && !quiet)) cerr <<
-        "- Default library routine search path used (GDL_PATH/IDL_PATH env. var. not set): " << endl << 
-        "  " << gdlPath << endl;
+        "- Default library routine search path used (GDL_PATH/IDL_PATH env. var. not set): " GDLDATADIR "/lib" << endl;
+    }
+  if (useWxWidgetsForGraphics) {
+      if (gdlde || (isatty(0) && !quiet)) cerr << "- Using WxWidgets as graphics library (windows and widgets)." << endl;
     }
 
   
   if (useDSFMTAcceleration && (GetEnvString("GDL_NO_DSFMT").length() > 0)) useDSFMTAcceleration=false;
-
-  SysVar::SetGDLPath( gdlPath);
+  
+  //report in !GDL status struct
+  DStructGDL* gdlconfig = SysVar::GDLconfig();
+  unsigned  DSFMTTag= gdlconfig->Desc()->TagIndex("GDL_USE_DSFMT");
+  (*static_cast<DByteGDL*> (gdlconfig->GetTag(DSFMTTag, 0)))[0]=useDSFMTAcceleration;
+  
+  //same for use of wxwidgets
+  unsigned  useWXTAG= gdlconfig->Desc()->TagIndex("GDL_USE_WX");
+  (*static_cast<DByteGDL*> (gdlconfig->GetTag(useWXTAG, 0)))[0]=useWxWidgetsForGraphics;
+  
+  SysVar::SetGDLPath( gdlPath); //probably duplicate with the one in initobjects!
   
   if (!pretendRelease.empty()) SysVar::SetFakeRelease(pretendRelease);
   //fussyness setup and change if switch at start
@@ -416,8 +479,7 @@ int main(int argc, char *argv[])
 
   if (gdlde || (isatty(0) && !quiet))
   {
-    cerr << "- Please report bugs, feature or help requests and patches at:" << endl <<
-      "  https://github.com/gnudatalanguage/gdl" << endl << endl;
+    cerr << "- Please report bugs, feature or help requests and patches at: https://github.com/gnudatalanguage/gdl" << endl << endl;
   }
 //   else
 //     {
