@@ -49,7 +49,9 @@ using namespace std;
 
 // instantiate the global lists
 VarListT      sysVarList;
+VarListT      obsoleteSysVarList; //spceial case of very very old sysvar, some point to [parts of] values in sysVar (e.g. !CXMAX): cannot be destroyed in a .RESET (double free())
 VarListT      sysVarRdOnlyList;
+VarListT      sysVarNoSaveList;
 
 FunListT      funList;
 ProListT      proList;
@@ -81,20 +83,30 @@ namespace structDesc {
   DStructDesc* GDL_CONTAINER = NULL;
   DStructDesc* GDL_CONTAINER_NODE = NULL;
   DStructDesc* GDL_HASHTABLEENTRY = NULL;
+  DStructDesc* IDLFFSHAPE = NULL;
+  DStructDesc* IDLFFXMLSAX = NULL;
 }
 
 // for OpenMP
 DLong CpuTPOOL_NTHREADS;
 DLong64 CpuTPOOL_MIN_ELTS;
 DLong64 CpuTPOOL_MAX_ELTS;
-const DLong64 CpuTPOOL_MAX_ELTS_max = numeric_limits<DLong64>::max();
+//const DULong64 CpuTPOOL_MAX_ELTS_max = numeric_limits<DLong64>::max();
 
 // instantiate own AST factory
 //_DNodeFactory DNodeFactory;
 antlr::ASTFactory DNodeFactory("DNode",DNode::factory);
 
+//do we use WxWidgets at all?
+volatile bool useWxWidgets;
 //do we use WxWidgets for graphics?
 volatile bool useWxWidgetsForGraphics;
+//do we use SVG for graphics in python notebook use?
+volatile bool iAmANotebook;
+//do we use name Devices differently among platforms?
+volatile bool usePlatformDeviceName;
+// do we force fonts to be the ugly IDL fonts?
+volatile bool forceWxWidgetsUglyFonts;
 
 //do we favor SIMD-accelerated random number generation?
 volatile bool useDSFMTAcceleration;
@@ -154,6 +166,7 @@ void InitStructs()
   SpDInt   aColor( dimension(3));
   SpDLong   aLongArr8( dimension(8));
   SpDLong64   aLong64Arr8( dimension(8));
+  SpDDouble   aDoubleArr8( dimension(8));
   SpDPtr    aPtrRef;
   SpDObj    aObjRef;
   SpDUInt   auInt;
@@ -186,9 +199,9 @@ void InitStructs()
 
 
   DStructDesc* gdl_object = new DStructDesc( GDL_OBJECT_NAME);
-  gdl_object->AddTag("GDL_OBJ_TOP", &aLong64);
+  gdl_object->AddTag("IDL_OBJECT_TOP", &aLong64);
   gdl_object->AddTag("__OBJ__", &aObjRef);
-  gdl_object->AddTag("GDL_OBJ_BOTTOM", &aLong64);
+  gdl_object->AddTag("IDL_OBJECT_BOTTOM", &aLong64);
   // special for GDL_OBJECT ony
   gdl_object->InitOperatorList();
   // insert into structList
@@ -217,12 +230,12 @@ void InitStructs()
   structDesc::GDL_CONTAINER_NODE = gdlContainerNode;
 
   DStructDesc* gdlContainer = new DStructDesc( GDL_CONTAINER_NAME);
-  gdlContainer->AddTag("GDL_CONTAINER_TOP", &aLong64);
-  gdlContainer->AddTag("GDLCONTAINERVERSION", &aInt);
+  gdlContainer->AddTag("IDL_CONTAINER_TOP", &aLong64);
+  gdlContainer->AddTag("IDLCONTAINERVERSION", &aInt);
   gdlContainer->AddTag("PHEAD", &aPtrRef);
   gdlContainer->AddTag("PTAIL", &aPtrRef);
   gdlContainer->AddTag("NLIST", &aLong);
-  gdlContainer->AddTag("GDL_CONTAINER_BOTTOM", &aLong64);
+  gdlContainer->AddTag("IDL_CONTAINER_BOTTOM", &aLong64);
 //  gdlContainer->AddParent(gdl_object);// no operator overloading
   structList.push_back(gdlContainer);
   structDesc::GDL_CONTAINER = gdlContainer;
@@ -246,6 +259,65 @@ void InitStructs()
   structList.push_back(gdlHashTE);
   structDesc::GDL_HASHTABLEENTRY = gdlHashTE;
   
+#ifdef USE_SHAPELIB  
+  //for IDLffShape
+  DStructDesc* gdlffShape = new DStructDesc( "IDLFFSHAPE");
+  gdlffShape->AddTag("IDLFFSHAPE_TOP", &aLong64);
+  gdlffShape->AddTag("IDLFFSHAPEVERSION", &aInt);
+  gdlffShape->AddTag("FILENAME", &aString);
+  gdlffShape->AddTag("ISOPEN", &aInt);
+  gdlffShape->AddTag("SHPTYPE", &aInt);
+  gdlffShape->AddTag("PATTRIBUTE", &aPtrRef);
+  gdlffShape->AddTag("SHAPEHANDLE", &aLong64);
+  gdlffShape->AddTag("DBFHANDLE", &aLong64);
+  gdlffShape->AddTag("IDLFFSHAPE_BOTTOM", &aLong64);
+  // insert into structList
+  structList.push_back(gdlffShape);
+  structDesc::IDLFFSHAPE = gdlffShape;
+ 
+  
+  DStructDesc* gdlffShape_entity = new DStructDesc( "IDL_SHAPE_ENTITY");
+  gdlffShape_entity->AddTag("SHAPE_TYPE", &aLong);
+  gdlffShape_entity->AddTag("ISHAPE", &aLong);
+  gdlffShape_entity->AddTag("BOUNDS", &aDoubleArr8);
+  gdlffShape_entity->AddTag("N_VERTICES", &aLong);
+  gdlffShape_entity->AddTag("VERTICES", &aPtrRef);
+  gdlffShape_entity->AddTag("MEASURE", &aPtrRef);
+  gdlffShape_entity->AddTag("N_PARTS", &aLong);
+  gdlffShape_entity->AddTag("PARTS", &aPtrRef);
+  gdlffShape_entity->AddTag("PART_TYPES", &aPtrRef);
+  gdlffShape_entity->AddTag("ATTRIBUTES", &aPtrRef);
+  // insert into structList
+  structList.push_back(gdlffShape_entity);
+  
+  DStructDesc* gdlffShape_attribute = new DStructDesc( "IDL_SHAPE_ATTRIBUTE");
+  gdlffShape_attribute->AddTag("NAME", &aString);
+  gdlffShape_attribute->AddTag("TYPE", &aLong);
+  gdlffShape_attribute->AddTag("WIDTH", &aLong);
+  gdlffShape_attribute->AddTag("PRECISION", &aLong);
+  // insert into structList
+  structList.push_back(gdlffShape_attribute);
+#endif
+  
+#ifdef USE_EXPAT
+  
+  //for IDLffXMLSAX
+  DStructDesc* gdlffXmlSax = new DStructDesc( "IDLFFXMLSAX");
+  gdlffXmlSax->AddTag("IDLFFXMLSAX_TOP", &aLong64);
+  gdlffXmlSax->AddTag("IDLFFXMLSAXVERSION", &aInt);
+  gdlffXmlSax->AddTag("VALIDATION_MODE", &aInt);
+  gdlffXmlSax->AddTag("HALT_PROCESSING", &aInt);
+  gdlffXmlSax->AddTag("_XML_PARSER", &aLong64);
+  gdlffXmlSax->AddTag("_XML_LOCATOR", &aLong64);
+  gdlffXmlSax->AddTag("IDLFFXMLSAX_BOTTOM", &aLong64);
+//  gdlffXmlSax->AddParent(gdl_object);
+  // insert into structList
+  structList.push_back(gdlffXmlSax);
+  structDesc::IDLFFXMLSAX = gdlffXmlSax;
+#endif
+  
+  
+  
   // OBJECTS END =======================================================
    
   for (int big = 1; big >= 0; --big) 
@@ -388,7 +460,6 @@ void InitStructs()
   widgdestroy->AddTag("ID", &aLong);
   widgdestroy->AddTag("TOP", &aLong);
   widgdestroy->AddTag("HANDLER", &aLong);
-  widgdestroy->AddTag("MESSAGE", &aLong);
   // insert into structList
   structList.push_back( widgdestroy);
   
@@ -397,9 +468,18 @@ void InitStructs()
   toplevelISdestroyed->AddTag("ID", &aLong);
   toplevelISdestroyed->AddTag("TOP", &aLong);
   toplevelISdestroyed->AddTag("HANDLER", &aLong);
-  toplevelISdestroyed->AddTag("MESSAGE", &aLong);
   // insert into structList
   structList.push_back( toplevelISdestroyed);
+
+  //for internal usage: event structure used to pass any catched "C++" event (as in ThrowGDLException)
+  //back to $MAIN$ to process as a normal error like in e->Throw();
+  DStructDesc* WidRunTimeError = new DStructDesc( "*WIDGET_RUNTIME_ERROR*");
+  WidRunTimeError->AddTag("ID", &aLong);
+  WidRunTimeError->AddTag("TOP", &aLong);
+  WidRunTimeError->AddTag("HANDLER", &aLong);
+  WidRunTimeError->AddTag("MESSAGE", &aString);
+  // insert into structList
+  structList.push_back( WidRunTimeError);
   
   DStructDesc* widgbut = new DStructDesc( "WIDGET_BUTTON");
   widgbut->AddTag("ID", &aLong);
