@@ -23,14 +23,13 @@ namespace lib {
 
   using namespace std;
 
-  void window( EnvT* e)
-  {
+  void window(EnvT* e) {
     GraphicsDevice* actDevice = GraphicsDevice::GetDevice();
     int maxWin = actDevice->MaxWin();
-    if( maxWin == 0)
-      e->Throw( "Routine is not defined for current graphics device.");
+    if (maxWin == 0)
+      e->Throw("Routine is not defined for current graphics device.");
 
-    SizeT nParam=e->NParam();
+    SizeT nParam = e->NParam();
 
     DLong wIx = 0;
     static int FREEIx = e->KeywordIx("FREE");
@@ -41,79 +40,69 @@ namespace lib {
     static int YSIZEIx = e->KeywordIx("YSIZE");
     static int RETAINIx = e->KeywordIx("RETAIN");
     static int PIXMAPIx = e->KeywordIx("PIXMAP");
-    if( e->KeywordSet(FREEIx))
-      {
-	wIx = actDevice->WAddFree();
-	if( wIx == -1)
-	  e->Throw( "No more window handles left.");
-      }
-    else
-      {
-	if( nParam == 1)
-	  {
-	    e->AssureLongScalarPar( 0, wIx);
-	    if( wIx < 0 || wIx >= actDevice->MaxNonFreeWin()) //to comply with IDL
-	      e->Throw( "Window number "+i2s(wIx)+
-			" out of range or no more windows.");
-	  }
+    if (e->KeywordSet(FREEIx)) {
+      wIx = actDevice->WAddFree();
+      if (wIx == -1)
+        e->Throw("No more window handles left.");
+    } else {
+      if (nParam == 1) {
+        e->AssureLongScalarPar(0, wIx);
+        if (wIx < 0 || wIx >= actDevice->MaxNonFreeWin()) //to comply with IDL
+          e->Throw("Window number " + i2s(wIx) +
+          " out of range or no more windows.");
       }
+    }
 
     DString title;
-    if( e->KeywordPresent(TITLEIx))
-      {
-	e->AssureStringScalarKWIfPresent( TITLEIx, title);
-      }
-    else
-      {
-	title = "GDL "+i2s( wIx);
-      }
+    if (e->KeywordPresent(TITLEIx)) {
+      e->AssureStringScalarKWIfPresent(TITLEIx, title);
+    } else {
+      title = "GDL " + i2s(wIx);
+    }
 
-    DLong xPos=-1, yPos=-1; //NOTE: xPos=-1 and yPos=-1 are when XPOS and YPOS options were not used!
-    e->AssureLongScalarKWIfPresent( XPOSIx, xPos);
-    e->AssureLongScalarKWIfPresent( YPOSIx, yPos);
+    DLong xPos = -1, yPos = -1; //NOTE: xPos=-1 and yPos=-1 are when XPOS and YPOS options were not used!
+    e->AssureLongScalarKWIfPresent(XPOSIx, xPos);
+    e->AssureLongScalarKWIfPresent(YPOSIx, yPos);
 
     DLong xSize, ySize;
 
     actDevice->DefaultXYSize(&xSize, &ySize);
 
-    e->AssureLongScalarKWIfPresent( XSIZEIx, xSize);
-    e->AssureLongScalarKWIfPresent( YSIZEIx, ySize);
+    e->AssureLongScalarKWIfPresent(XSIZEIx, xSize);
+    e->AssureLongScalarKWIfPresent(YSIZEIx, ySize);
 
-    int debug=0;
+    int debug = 0;
     if (debug) {
-      cout << "input values :"<< endl;
-      cout << "xPos/yPos   :"<<  xPos << " " << yPos << endl;
-      cout << "xSize/ySize :"<<  xSize << " " << ySize << endl;      
+      cout << "input values :" << endl;
+      cout << "xPos/yPos   :" << xPos << " " << yPos << endl;
+      cout << "xSize/ySize :" << xSize << " " << ySize << endl;
     }
-    
-    if (xSize < 0) xSize=10000;
-    if (ySize < 0) ySize=10000;
+
+    if (xSize < 0) xSize = 10000;
+    if (ySize < 0) ySize = 10000;
 
 
-//NOTE: xPos=-1 and yPos=-1 are when XPOS and YPOS options were not used!   
+    //NOTE: xPos=-1 and yPos=-1 are when XPOS and YPOS options were not used!   
 
-// AC 2014/02/26: this seems to be not need as long as xSize/ySize > 0
-// which is done above
+    // AC 2014/02/26: this seems to be not need as long as xSize/ySize > 0
+    // which is done above
 
-//    if( xSize <= 0 || ySize <= 0 || xPos < -1 || yPos < -1)
-//      e->Throw(  "Unable to create window "
-//		 "(BadValue (integer parameter out of range for operation)).");
-    
-    DLong retainType = 0; 
-    if( e->KeywordPresent( RETAINIx))
-    {
-      e->AssureLongScalarKWIfPresent( RETAINIx, retainType);
+    //    if( xSize <= 0 || ySize <= 0 || xPos < -1 || yPos < -1)
+    //      e->Throw(  "Unable to create window "
+    //		 "(BadValue (integer parameter out of range for operation)).");
+
+    DLong retainType = 0;
+    if (e->KeywordPresent(RETAINIx)) {
+      e->AssureLongScalarKWIfPresent(RETAINIx, retainType);
     }
     bool success = actDevice->SetBackingStore(retainType);
-    bool hide=e->KeywordSet( PIXMAPIx);
-    success = actDevice->WOpen( wIx, title, xSize, ySize, xPos, yPos, hide);
-    if( !success)
-      e->Throw(  "Unable to create window.");
-    if ( e->KeywordSet( PIXMAPIx))
-    {
+    bool hide = e->KeywordSet(PIXMAPIx);
+    success = actDevice->WOpen(wIx, title, xSize, ySize, xPos, yPos, hide);
+    if (!success)
+      e->Throw("Unable to create window.");
+    if (e->KeywordSet(PIXMAPIx)) {
       success = actDevice->Hide();
-    }
-    else success = actDevice->UnsetFocus();
+    } else success = actDevice->UnsetFocus();
     actDevice->GetStream()->DefaultBackground();
     actDevice->GetStream()->Clear();
 
@@ -158,16 +147,15 @@ namespace lib {
       e->Throw("Window is closed and unavailable.");
   }
 
-  void wshow( EnvT* e)
-  {
+  void wshow(EnvT* e) {
     GraphicsDevice* actDevice = GraphicsDevice::GetDevice();
     int maxWin = actDevice->MaxWin();
-    if( maxWin == 0)
-      e->Throw( "Routine is not defined for current graphics device.");
+    if (maxWin == 0)
+      e->Throw("Routine is not defined for current graphics device.");
 
-    SizeT nParam=e->NParam();
+    SizeT nParam = e->NParam();
     DLong wIx = 0;
-    if (nParam != 0) e->AssureLongScalarPar( 0, wIx);
+    if (nParam != 0) e->AssureLongScalarPar(0, wIx);
     else wIx = actDevice->ActWin();
 
     // note by AC on 2012-Aug-16
@@ -176,48 +164,45 @@ namespace lib {
     // Help/suggestions welcome
     // works for me (GD) on mandriva 2010.2
     bool show = true;
-    if (nParam == 2) { 
+    if (nParam == 2) {
       DIntGDL *showval = e->GetParAs<DIntGDL>(1);
       show = (*showval)[0] != 0;
     }
 
     int iconic = -1; //signals absent parameter
     static int ICONICIx = e->KeywordIx("ICONIC");
-    if( e->KeywordPresent(ICONICIx)) iconic=e->KeywordSet(ICONICIx);
+    if (e->KeywordPresent(ICONICIx)) iconic = e->KeywordSet(ICONICIx);
 
-    if (!actDevice->WShow( wIx, show, iconic)) 
-      e->Throw( "Window number "+i2s(wIx)+" out of range or no more windows.");
+    if (!actDevice->WShow(wIx, show, iconic))
+      e->Throw("Window number " + i2s(wIx) + " out of range or no more windows.");
   }
 
-  void wdelete( EnvT* e)
-  {
+  void wdelete(EnvT* e) {
     GraphicsDevice* actDevice = GraphicsDevice::GetDevice();
     int maxWin = actDevice->MaxWin();
-    if( maxWin == 0)
-      e->Throw( "Routine is not defined for current graphics device.");
+    if (maxWin == 0)
+      e->Throw("Routine is not defined for current graphics device.");
 
-    SizeT nParam=e->NParam();
-    if( nParam == 0)
-      {
-	DLong wIx = actDevice->ActWin();
-	bool success = actDevice->WDelete( wIx);
-	if( !success)
-	  e->Throw( "Window number "+i2s(wIx)+
-			      " invalid or no more windows.");
-	return;
-      }
+    SizeT nParam = e->NParam();
+    if (nParam == 0) {
+      DLong wIx = actDevice->ActWin();
+      bool success = actDevice->WDelete(wIx);
+      if (!success)
+        e->Throw("Window number " + i2s(wIx) +
+        " invalid or no more windows.");
+      return;
+    }
 
-    for( SizeT i=0; i<nParam; i++)
-      {
-	DLong wIx;
-	e->AssureLongScalarPar( i, wIx);
-	bool success = actDevice->WDelete( wIx);
-	if( !success)
-	  e->Throw( "Window number "+i2s(wIx)+
-		    " invalid or no more windows.");
-      }
+    for (SizeT i = 0; i < nParam; i++) {
+      DLong wIx;
+      e->AssureLongScalarPar(i, wIx);
+      bool success = actDevice->WDelete(wIx);
+      if (!success)
+        e->Throw("Window number " + i2s(wIx) +
+        " invalid or no more windows.");
+    }
   }
 
-    
+
 
 } // namespace
